package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.core.oe.admin.api.bean.MarkResultScoreBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.ClearMarkResultReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.SaveSubjectiveQuestionScoreReq;
import cn.com.qmth.examcloud.core.oe.admin.dao.MarkResultRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.MarkResultEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.admin.service.MarkResultService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.SubjectiveQuestionScoreInfo;
import cn.com.qmth.examcloud.marking.api.bean.SubjectiveScoreBean;

@Service
public class MarkResultServiceImpl implements MarkResultService {

    @Autowired
    private MarkResultRepo markResultRepo;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Transactional
    @Override
    public void modifyScore(SaveSubjectiveQuestionScoreReq req) {
        markResultRepo.deleteByExamRecordDataId(req.getExamRecordDataId());
        markResultRepo.flush();
        List<MarkResultEntity> ret = new ArrayList<>();
        for (MarkResultScoreBean dto : req.getScores()) {
            MarkResultEntity tem = new MarkResultEntity();
            tem.setRootOrgId(req.getRootOrgId());
            tem.setCourseId(req.getCourseId());
            tem.setExamId(req.getExamId());
            tem.setExamRecordDataId(req.getExamRecordDataId());
            tem.setWorkId(req.getWorkId());
            tem.setStudentPaperId(req.getStudentPaperId());
            tem.setMainNumber(dto.getMainNumber());
            tem.setOrders(dto.getOrders());
            tem.setQuestionId(dto.getQuestionId());
            tem.setScore(dto.getScore());
            tem.setSubNumber(dto.getSubNumber());
            ret.add(tem);
        }
        markResultRepo.saveAll(ret);
        saveSubjectiveQuestionScore(req);
    }

    private void saveSubjectiveQuestionScore(SaveSubjectiveQuestionScoreReq req) {
        List<SubjectiveQuestionScoreInfo> subjectiveQuestionScoreInfoList = new ArrayList<SubjectiveQuestionScoreInfo>();
        for (MarkResultScoreBean bean : req.getScores()) {
            SubjectiveQuestionScoreInfo info = new SubjectiveQuestionScoreInfo();
            info.setExamRecordDataId(req.getExamRecordDataId());
            info.setOrder(bean.getOrders());
            info.setQuestionId(bean.getQuestionId());
            info.setScore(bean.getScore());
            subjectiveQuestionScoreInfoList.add(info);
        }
        examRecordQuestionsService.saveSubjectiveQuestionScore(req.getExamRecordDataId(),
                subjectiveQuestionScoreInfoList);

    }

    @Transactional
    @Override
    public void clearScore(ClearMarkResultReq req) {
        markResultRepo.deleteByExamRecordDataId(req.getExamRecordDataId());
    }

    @Override
    public List<SubjectiveScoreBean> getScores(Long examRecordDataId) {
        List<MarkResultEntity> es = markResultRepo.findByExamRecordDataId(examRecordDataId);
        if (CollectionUtils.isEmpty(es)) {
            return null;
        }
        List<SubjectiveScoreBean> ret = new ArrayList<>();
        for (MarkResultEntity e : es) {
            SubjectiveScoreBean b = new SubjectiveScoreBean();
            ret.add(b);
            b.setQuestionOrder(e.getOrders());
            b.setScore(e.getScore());
        }
        return ret;
    }
}
